﻿using System;
using ETHZ.C0284H.Calculator.Instruct.Commands;
using ETHZ.C0284H.Calculator.Instruct.Operators;

namespace ETHZ.C0284H.Calculator.Instruct.Visitor
{
    /// <summary>
    ///  Calculator executor operates on calculator on behalf of a controller.
    /// </summary>
    internal class CalculatorExecutor : ICalculatorInstructVisitor
    {
        public CalculatorExecutor(ReversePolishNotationCalculator calculator)
        {
            if (calculator == null)
            {
                throw new ArgumentNullException("calculator");
            }

            Calculator = calculator;
        }

        /// <summary>
        ///     Executor indicating whether last visit processed the instruct.
        /// </summary>
        public bool InstructProcessed { get; protected set; }

        /// <summary>
        ///     Calculator property.
        /// </summary>
        private ReversePolishNotationCalculator Calculator { get; set; }

        public void VisitCommand(QuitInstruct command)
        {
            InstructProcessed = false;
        }

        public void VisitCommand(LanguageInstruct command)
        {
            InstructProcessed = false;
        }

        public void VisitCommand(HelpInstruct command)
        {
            InstructProcessed = false;
        }

        public void VisitCommand(ClearInstruct command)
        {
            Calculator.Clear();
            InstructProcessed = true;
        }

        public void VisitCommand(EvaluateInstruct command)
        {
            Calculator.Evaluate();
            InstructProcessed = true;
        }

        public void VisitCommand(PrintInstruct command)
        {
            InstructProcessed = false;
        }

        public void VisitOperator(DivideByInstruct minusOperator)
        {
            Calculator.PushDivideByOperator();
            InstructProcessed = true;
        }

        /// <summary>
        ///     Visitor method.
        /// </summary>
        /// <param name="operand"></param>
        public void VisitOperand(CalculatorInstructOperand operand)
        {
            if (operand != null)
            {
                Calculator.PushOperand(operand.Value);
                InstructProcessed = true;
            }
        }

        public void VisitOperator(PlusInstruct plusOperator)
        {
            if (plusOperator != null)
            {
                Calculator.PushPlusOperator();
                InstructProcessed = true;
            }
        }

        public void VisitOperator(MinusInstruct minusOperator)
        {
            if (minusOperator != null)
            {
                Calculator.PushMinusOperator();
                InstructProcessed = true;
            }
        }

        public void VisitOperator(TimesInstruct timesOperator)
        {
            if (timesOperator != null)
            {
                Calculator.PushTimesOperator();
                InstructProcessed = true;
            }
        }
    }
}